@@ -4,15 +4,15 @@
 
    Author: Ilan Schnell
 */
-#define BITARRAY_VERSION  "2.8.4"
+#define BITARRAY_VERSION  "2.8.5"
 
 #ifdef STDC_HEADERS
-#include <stddef.h>
-#else  /* !STDC_HEADERS */
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>      /* For size_t */
-#endif /* HAVE_SYS_TYPES_H */
-#endif /* !STDC_HEADERS */
+#  include <stddef.h>
+#else
+#  ifdef HAVE_SYS_TYPES_H
+#    include <sys/types.h>      /* For size_t */
+#  endif
+#endif
 
 /* Compatibility with Visual Studio 2013 and older which don't support
    the inline keyword in C (only in C++): use __inline instead.
@@ -22,6 +22,10 @@
 #define inline __inline
 #endif
 
+#ifdef _MSC_VER
+#include <intrin.h>    /* For _byteswap_uint64() */
+#endif
+
 /* --- definitions specific to Python --- */
 
 /* Py_UNREACHABLE was introduced in Python 3.7 */
@@ -29,12 +33,6 @@
 #define Py_UNREACHABLE() abort()
 #endif
 
-/* PY_LITTLE_ENDIAN and PY_BIG_ENDIAN (machine byteorder) are available in
-   Python 3.4.10.  Not sure when exactly they were introduced. */
-#ifndef PY_LITTLE_ENDIAN
-#define PY_LITTLE_ENDIAN  (*((uint64_t *) "\xff\0\0\0\0\0\0\0") == 0xff)
-#endif
-
 #if PY_MAJOR_VERSION >= 3
 #define IS_PY3K  1
 #define BYTES_SIZE_FMT  "y#"
@@ -59,14 +57,14 @@ typedef struct {
     char *ob_item;              /* buffer */
     Py_ssize_t allocated;       /* allocated buffer size (in bytes) */
     Py_ssize_t nbits;           /* length of bitarray, i.e. elements */
-    int endian;                 /* bit endianness of bitarray */
+    int endian;                 /* bit-endianness of bitarray */
     int ob_exports;             /* how many buffer exports */
     PyObject *weakreflist;      /* list of weak references */
     Py_buffer *buffer;          /* used when importing a buffer */
     int readonly;               /* buffer is readonly */
 } bitarrayobject;
 
-/* --- bit endianness --- */
+/* --- bit-endianness --- */
 #define ENDIAN_LITTLE  0
 #define ENDIAN_BIG     1
 
@@ -196,7 +194,36 @@ popcnt_64(uint64_t x)
     x = (x & m2) + ((x >> 2) & m2);
     x = (x + (x >> 4)) & m4;
     return (x * h01) >> 56;
-#endif  /* __GNUC__ */
+#endif
+}
+
+static inline uint64_t
+builtin_bswap64(uint64_t word)
+{
+#if (defined(__clang__) ||                                                 \
+      (defined(__GNUC__)                                                   \
+        && ((__GNUC__ >= 5) || (__GNUC__ == 4) && (__GNUC_MINOR__ >= 3))))
+    /* __builtin_bswap64() is available since GCC 4.3. */
+#  define HAVE_BUILTIN_BSWAP64  1
+    return __builtin_bswap64(word);
+#elif defined(_MSC_VER)
+#  define HAVE_BUILTIN_BSWAP64  1
+    return _byteswap_uint64(word);
+#else
+#  define HAVE_BUILTIN_BSWAP64  0
+    Py_UNREACHABLE();
+#endif
+}
+
+/* Return distance [0..3] to next aligned pointer.
+   While on modern compilers uint64_t pointers may be misaligned, it may
+   cause problems on older ones.  Moreover, it may lead to slowdown (even
+   on modern compilers). */
+static inline int
+to_aligned(void *p)
+{
+    int r = ((uintptr_t) p) % 4;
+    return r ? 4 - r : 0;
 }
 
 /* population count of n words starting from at uint64_t pointer w */
@@ -205,7 +232,7 @@ popcnt_words(uint64_t *w, Py_ssize_t n)
 {
     Py_ssize_t cnt = 0;
 
-    assert(n >= 0);
+    assert(n >= 0 && to_aligned((void *) w) == 0);
     while (n--)
         cnt += popcnt_64(*w++);
     return cnt;
