@@ -31,8 +31,6 @@ from bitarray.util import (
 
 if DEBUG:
     from bitarray._util import _sc_rts, _SEGSIZE  # type: ignore
-else:
-    _SEGSIZE = -1
 
 if sys.version_info[0] == 3:
     from io import StringIO
@@ -1392,7 +1390,7 @@ class SC_Tests(unittest.TestCase, Util):
         self.assertEqual(len(rts), 1)
         self.assertEqual(rts, [0])
 
-    @skipIf(_SEGSIZE != 32)
+    @skipIf(not DEBUG or _SEGSIZE != 32)
     def test_rts_example(self):
         # see example before sc_calc_rts() in _util.c
         a = zeros(987)
@@ -1402,6 +1400,17 @@ class SC_Tests(unittest.TestCase, Util):
         self.assertEqual(len(rts), 5)
         self.assertEqual(rts, [0, 5, 5, 8, 12])
 
+    @skipIf(not DEBUG)
+    def test_rts_ones(self):
+        for n in range(2000):
+            a = bitarray(n)
+            a.setall(1)
+            rts = _sc_rts(a)
+            self.assertEqual(rts[0], 0)
+            self.assertEqual(rts[-1], n)
+            for i, v in enumerate(rts):
+                self.assertEqual(v, min(8 * _SEGSIZE * i, n))
+
     @skipIf(not DEBUG)
     def test_rts_random(self):
         segbits = 8 * _SEGSIZE
@@ -1411,7 +1420,6 @@ class SC_Tests(unittest.TestCase, Util):
             self.assertEqual(len(rts), (n + segbits - 1) // segbits + 1)
             self.assertEqual(rts[0], 0)
             self.assertEqual(rts[-1], a.count())
-
             for i in range(len(rts) - 1):
                 seg_pop = a.count(1, segbits * i, segbits * (i + 1))
                 self.assertEqual(rts[i + 1] - rts[i], seg_pop)
